package com.loafle.overflow.central.module.sensor.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.sensor.dao.SensorDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;
import com.loafle.overflow.service.central.sensor.SensorItemService;
import com.loafle.overflow.service.central.sensor.SensorService;
import com.loafle.overflow.service.central.target.TargetService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorService")
public class CentralSensorService implements SensorService {

  @Autowired
  SensorDAO sensorDAO;

  @Autowired
  private SensorItemService sensorItemService;

  @Autowired
  private TargetService targetService;

  @Transactional
  public Sensor regist(Sensor sensor, Long targetID) throws OverflowException {
    this.targetService.increaseSensorCount(targetID);
    return this.sensorDAO.save(sensor);
  }

  public Sensor read(Long id) throws OverflowException {
    return this.sensorDAO.findById(id).get();
  }

  @Transactional
  public void remove(Long id, Long targertID) throws OverflowException {
    this.targetService.decreaseSensorCount(targertID);
    this.sensorDAO.deleteById(id);
  }

  public Page<Sensor> readAllByProbeID(Long probeID, PageParams pageParams) throws OverflowException {
    // FIXME: NOT IMPLEMENTS
    return null;
  }

  public Page<Sensor> readAllByDomainID(Long domainID, PageParams pageParams) throws OverflowException {
    // FIXME: NOT IMPLEMENTS
    return null;
  }

  public Page<Sensor> readAllByInfraID(Long infraID, PageParams pageParams) throws OverflowException {
    return this.sensorDAO.findAllByTargetInfraId(infraID, PageUtil.getPageRequest(pageParams));
  }

  public Page<Sensor> readAllByTargetID(Long targetID, PageParams pageParams) throws OverflowException {
    return this.sensorDAO.findAllByTargetId(targetID, PageUtil.getPageRequest(pageParams));
  }

  @Override
  public Sensor increaseSensorItemCount(Long id) throws OverflowException {
    Sensor s = this.sensorDAO.findById(id).get();
    s.setItemCount(s.getItemCount() + 1);
    return this.sensorDAO.save(s);
  }

  @Override
  public Sensor decreaseSensorItemCount(Long id) throws OverflowException {
    Sensor s = this.sensorDAO.findById(id).get();
    int count = s.getItemCount();
    if (count > 0) {
      s.setItemCount(count - 1);
    }
    return this.sensorDAO.save(s);
  }

  @Transactional
  public Sensor registSensorConfig(Sensor sensor, List<SensorItem> sensorItemList, String etcJson)
      throws OverflowException {

    this.targetService.increaseSensorCount(sensor.getTarget().getId());
    this.sensorDAO.save(sensor);

    for (SensorItem sensorItem : sensorItemList) {
      sensorItem.setSensor(sensor);
    }

    this.sensorItemService.registAll(sensorItemList);

    return sensor;
  }

  @Override
  public String generateSensorConfig(Sensor sensor) throws OverflowException {
    return null;
  }

}